with Ada.Text_IO;
package body Museum is

   protected body Guardian is

      entry Enter
        when (Enter'Count = Group_Size or Release) and Is_Free is
      begin
         --  the first time we reach here
         --  it must be because ('Count = Group_Size)
         --  hence we must make sure that we do not allow
         --  more visitors in than the Museum regulations specify
         --+
         --  of course as soon as we let visitors in
         --  we have ('Count /= Group_Size)
         --  hence we must make sure that the value of Release
         --  keeps the guard open until the last visitor of the group
         --  has entered the museum
         if Enter'Count = 0 then
            Release := False;
            Inside := Inside + Group_Size;
         else
            Release := True;
         end if;
      end Enter;

      procedure Leave is
      begin
         Inside := Inside - 1;
      end Leave;

      function Is_Free return Boolean is
      begin
         return ((Inside + Group_Size) <= Capacity);
      end Is_Free;

   end Guardian;

begin
   Ada.Text_IO.Put_Line ("This museum has capacity "
                    & Capacity'Image
                    & " and entrance group size "
                    & Group_Size'Image);
end Museum;
